@@ -20,7 +20,7 @@
 	}
 	ret
 }
-overDF_for_rgeos = .overDF # to be exported, for rgeos only
+overDF_for_rgeos = .overDF # to be exported, for rgeos, and spacetime
 
 # we need to invert a list of indexes, i.e.
 # list(c(1,4), c(2,4,5))
@@ -52,11 +52,13 @@ overDF_for_rgeos = .overDF # to be exported, for rgeos only
 # when changing this function, we also might want to change
 # overGeomGeomDF in rgeos,
 # ... and overDFGenericST in spacetime.
-overDFGeneric = function(x, y, returnList = FALSE, fn = NULL, ...) {
+overDFGeneric = function(x, y, returnList = FALSE, fn = NULL, ..., minDimension = -1) {
 	stopifnot(identicalCRS(x, y))
-	r = over(x, geometry(y), returnList = TRUE)
+	r = over(x, geometry(y), returnList = TRUE, minDimension = minDimension)
 	ret = .overDF(r, y@data, length(x), returnList, fn, ...)
-	if (! returnList)
+	if (returnList)
+		names(ret) = row.names(x)
+	else
 		row.names(ret) = row.names(x)
 	ret
 }
@@ -75,6 +77,7 @@ setMethod("over",
 				ret = rep(as.integer(NA), length(x))
 				ret[zd[,1]] = zd[,2]
 			}
+			names(ret) = row.names(x)
 			ret
 		}
 )
@@ -85,6 +88,7 @@ setMethod("over",
 			r = pointsInSpatialPolygons(x, y, returnList)
 			if (returnList)
 				r = .invert(r, length(y), length(x))
+			names(r) = row.names(x)
 			r
 		}
 )
@@ -95,6 +99,7 @@ setMethod("over",
 			r = pointsInSpatialPolygons(geometry(y), geometry(x), TRUE)
 			if (!returnList)
 				r = sapply(r, function(x) x[1])
+			names(r) = row.names(x)
 			r
 		}
 )
@@ -127,7 +132,9 @@ setMethod("over", signature("SpatialPoints", "SpatialGrid"),
 	function(x, y, returnList = FALSE, fn = NULL, ...) {
 		stopifnot(identicalCRS(x,y))
 		idx = getGridIndex(coordinates(x), y@grid, all.inside = FALSE)
-		.index2list(idx, returnList)
+		r = .index2list(idx, returnList)
+		names(r) = row.names(x)
+		r
 	}
 )
 setMethod("over", signature("SpatialPoints", "SpatialPixels"), 
@@ -135,7 +142,9 @@ setMethod("over", signature("SpatialPoints", "SpatialPixels"),
 		stopifnot(identicalCRS(x,y))
 		idx = getGridIndex(coordinates(x), y@grid, all.inside = FALSE)
 		idx = match(idx, y@grid.index)
-		.index2list(idx, returnList)
+		r = .index2list(idx, returnList)
+		names(r) = row.names(x)
+		r
 	}
 )
 setMethod("over",
@@ -194,6 +203,39 @@ setMethod("over", signature("SpatialPoints", "SpatialPixelsDataFrame"),
 	}
 )
 
+setMethod("over", signature("Spatial", "Spatial"),  # catch remaining:
+	function(x, y, returnList = FALSE, fn = NULL, ...) {
+    	if (!requireNamespace("rgeos", quietly = TRUE))
+			stop("package rgeos is required for additional over methods")
+		if (is(x, "SpatialMultiPoints") || is(y, "SpatialMultiPoints"))
+			overMultiPoints(x, y, returnList = returnList, fn = fn, ...)
+		else
+			over(x, y, returnList = returnList, fn = fn, ...) # rgeos methods
+	}
+)
+
+overMultiPoints = function(x, y, returnList, fn, ...) {
+	if (is(x, "SpatialMultiPoints")) {
+		x = as(x, "SpatialPoints")
+		dimnames(x@coords)[[1]] = attr(x@coords, "groupIndex") # rgeos abuse
+	}
+	if (is(y, "SpatialMultiPoints")) {
+		if (is(y, "SpatialMultiPointsDataFrame")) {
+			yy = as(y, "SpatialPointsDataFrame")
+			yy@data = y@data  # strong sp abuse - yy no longer validates!
+			y = yy
+		} else
+			y = as(y, "SpatialPoints")
+		dimnames(y@coords)[[1]] = attr(y@coords, "groupIndex") # rgeos abuse
+	}
+   	if (!requireNamespace("rgeos", quietly = TRUE))
+		stop("package rgeos is required for additional over methods")
+	if ("data" %in% slotNames(y))
+		rgeos::overGeomGeomDF(x, y, returnList = returnList, fn = fn, ...)
+	else
+		rgeos::overGeomGeom(x, y, returnList = returnList, fn = fn, ...)
+}
+
 .index2list = function(x, returnList) {
 	if (returnList) {
 		l = lapply(1:length(x), function(x) { integer(0) })
@@ -203,11 +245,3 @@ setMethod("over", signature("SpatialPoints", "SpatialPixelsDataFrame"),
 	} else
 		x
 }
-
-aggregate.Spatial = function(x, by, FUN = mean, ...) {
-	by0 = by
-	if (gridded(by))
-		by = as(by, "SpatialPolygons")
-	df = over(by, x, fn = FUN, ...)
-	addAttrToGeom(by0, df, match.ID = FALSE)
-}
